/**
 * @file okolib.h
 * @mainpage Introduction
 * @tableofcontents
 *
 * This documentation describes the SDK specification used to operate devices
 * manufactured by <a href="http://www.oko-lab.com/" target="_blank">Okolab</a>.
 * The SDK will include an example C/C++ file showing typical usage and a description
 * of the API interface with the Okolab devices.
 *
 * @section intr1 Files
 * The API consists of a header file and libraries to be included in your project.
 * To access the device you have to call functions inside okolib.dll, which should reside
 * in your application directory.
 *
 * The SDK consists of the following files:
 * 		- Headers:  okolib.h
 * 		- Binaries: okolib.dll, libserialport.dll
 * 		- Resources: okolib.lib, okolib.db
 *
 * @section intr2 Concepts
 *
 * @subsection intr21 Device
 * A device is strictly related to a communication port and it is made up
 * by one or more physical controller units (e.g. Bold Line stacked units).
 * A device can be manually (@ref oko_DeviceOpen) or automatically (@ref oko_DevicesDetect) opened.
 *
 * @subsection intr22 Property
 * Each device has several properties.
 * Each property has some information (@ref PropInfo) and can be read or write.
 *
 * @section intr3 Usage
 * The operations that are supported are:
 *
 * - @ref Library (prepare the library to be used and query the version)
 * - @ref Open (prepare a device to be used)
 * - @ref Device (query status of the device)
 * - @ref PropInfo (for the selected device)
 * - @ref PropAuto
 * - @ref PropRead
 * - @ref PropWrite
 *
 * See also:
 * @ref usage1
 *
 * @section intr4 Information
 * @author Marco Di Pasqua <dipasqua@oko-lab.com>
 * @author Giulio Pasquariello <pasquariello@oko-lab.com>
 * @author Domenico Mastronardi
 * @copyright Okolab S.r.l. <a href="http://www.oko-lab.com/" target="_blank">oko-lab.com</a> <software.support@oko-lab.com>
 * @copyright This software uses code of <a href="http://sigrok.org/wiki/Libserialport" target="_blank">libserialport</a>
 * licensed under the <a href="https://www.gnu.org/licenses/lgpl-3.0.html" target="_blank">GNU LGPL, version 3</a>
 * and its source can be downloaded <a href="http://sigrok.org/download/source/libserialport/libserialport-0.1.0.tar.gz">here</a>
 *
 */

/** @page usage1 Typical Implementation
 *
 * @tableofcontents
 *
 * A typical implementation of the communication with an
 * Okolab device using the SDK would consist of a sequence
 * of function calls depending on the overall scenario.
 *
 * @section step1 Device initialization
 * @subsection case11 Case 1
 * No previous knowledge of the connected device
 * - @ref oko_DevicesDetect  Obtain the number of device connected and their handles as an array. For each handle
 *   + @ref oko_DeviceGetPortName  Store the port name to avoid going through the detect procedure next time software is started
 *
 * @subsection case12 Case 2
 * Previous knowledge of the device \e ComPort# (e.g. by user input or previously stored by the application).
 * - @ref oko_DeviceOpen  Obtain the handle of the device connected to \e ComPort#.
 *
 *
 * @section step2 Properties initialization
 *   + @ref oko_PropertiesGetNumber  Get the total number of properties for the current device
 *   + at this point the application should iterate calling @ref oko_PropertyGetName to create an array of \e PropertyNames. For each \e PropertyName
 *     + @ref oko_PropertyIsMain can be used to store main features only
 *     + @ref oko_PropertyGetType the @ref oko_prop_type of this property will be used to call the appropriate read/write function must be called
 *     + @ref oko_PropertyGetReadOnly when this returns false the property can also be write, if it can be write
 *       + @ref oko_PropertyHasLimits verify if the min and max of the property are available, if so
 *         + @ref oko_PropertyGetLimits read the min and max of the property from the device
 *     + @ref oko_PropertyGetDescription  optional, used to obtain property additional info
 *     + @ref oko_PropertyGetUnit  optional, used to obtain property additional info
 *
 *
 * @section step3 Property usage by Name
 * @subsection case31 Case 1
 * Get the last value of a read only property
 *   + @ref oko_PropertyGetType check the property type (e.g. @ref OKO_PROP_DOUBLE)
 *   + call the appropriate function @ref oko_PropertyReadDouble
 *
 * @subsection case32 Case 2
 * First update the value from the device and then read a property value of a known type
 *   + @ref oko_PropertyUpdate
 *   + @ref oko_PropertyReadDouble
 *
 * @subsection case33 Case 3
 * Synchronously change the value of a property (e.g. with double type)
 *   + @ref oko_PropertyWriteDouble with async = 0
 *
 * @subsection case34 Case 4
 * Change the value of a property (e.g. with double type), without waiting for the result
 *   + @ref oko_PropertyWriteDouble with async = 1, to avoid delay in the calling application execution
 *
 * @section step4 Auto-updating a property
 * A property can be auto-updated by an internal thread, so the application
 * doesn't need to esplicity update it, calling @ref oko_PropertyUpdate.
 *
 * @subsection case41 Case 1
 * Make a property auto-updating
 *   + @ref oko_PropertyAutoUpdate
 *
 * @subsection case42 Case 2
 * Read the last property value
 *   + @ref oko_PropertyReadDouble
 *
 *
 * @section step5 Close Device
 * @subsection case51 Case 1
 * No previous knowledge of the connected device
 * - @ref oko_DevicesCloseAll    Close the handles to all detected devices.
 *
 * @subsection case52 Case 2
 * Previous knowledge of the device
 * - @ref oko_DeviceClose close the device related to the specified handle.
 **/

#ifndef _OKOLIB_H_
#define _OKOLIB_H_

#ifdef _MSC_VER
    #if _MSC_VER >= 1600
        #include <stdint.h>
    #else
        typedef __int8              int8_t;
        typedef __int16             int16_t;
        typedef __int32             int32_t;
        typedef __int64             int64_t;
        typedef unsigned __int8     uint8_t;
        typedef unsigned __int16    uint16_t;
        typedef unsigned __int32    uint32_t;
        typedef unsigned __int64    uint64_t;
    #endif
#elif __GNUC__ >= 3
	#include <stdint.h>
#endif




#ifdef __cplusplus
extern "C" {
#endif

#ifdef _WIN32

#if defined(OKO_NO_DLL)             /* Static import */
#  define OKO_API
#elif defined(OKO_BUILDING_DLL)     /* DLL export    */
#  define OKO_API __declspec (dllexport)
#else                               /* DLL import    */
#  define OKO_API __declspec (dllimport)
#endif /* Not BUILDING_DLL */

/* Define calling convention in one place, for convenience. */
#define OKO_EXPORT __cdecl

#else /* _WIN32 not defined. */

/* Define with no value on non-Windows OSes. */
#define OKO_API
#define OKO_EXPORT

#endif

/**
@defgroup Errors Error codes
@{
*/


/** Specifies the return values */
typedef enum oko_res_type
{
	OKO_OK 				=  0,	//!< Operation completed successfully.
	OKO_ERR_UNINIT 		= -1,	//!< Library not initialized yet.
	OKO_ERR_ARG 		= -2,	//!< Invalid arguments were passed to the function.
	OKO_ERR_FAIL 		= -3,	//!< A system error occurred while executing the operation.
	OKO_ERR_NOTSUPP 	= -4,	//!< The requested operation is not supported by this system or device.
	OKO_ERR_CLOSED 		= -5,	//!< The specified device is not opened.
	OKO_ERR_UNCONN 		= -6,	//!< The specified device is not connected.
	OKO_DEV_SP_BUSY  	= -7,  //!< Serial post busy
	OKO_ERR_SP_CFG  	= -8, 	//!< Serial port configuration failed
	OKO_ERR_SP_SPEED  	= -9, 	//!< Serial port speed settings failed
	OKO_ERR_DB_OPEN  	= -10, 	//!< Database error on open
	OKO_PROP_NOTFOUND 	= -11, 	//!< Property not found
	OKO_DEV_NOTFOUND   	= -12,  //!< Device not found
	OKO_ERR_PROTOCOL   	= -13,  //!< Protocol error
	OKO_ERR_UNDEF  		= -999, //!< Undefined error
} oko_res_type;

/**
 * @}
 * @defgroup Library Library access functions
 * @{
*/

/*!
 * Initialize the library
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_LibInit();

/*!
 * Deallocates any resource that were allocated by oko_LibInitialize
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_LibShutDown();

/*!
 * Print the version of Okolib library on a string passed by reference.
 * @param[out] version	A pointer to a valid string where the version will be stored in.
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_LibGetVersion(char *version);

/**
 * @}
@defgroup Open Opening, closing and detecting devices
 * @{
*/

/*!
 * Obtains the number of available serial ports
 * @param[out] count the desired value
 * @return  OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_LibGetNumberOfPorts(uint32_t *count);

/*!
 * Obtains the avaialble port names
 * @param[out] names array of names (it must be already allocated)
 * @return  OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_LibGetPortNames(char **names);

/*!
 * Open the specified device, returning a handle for it.
 * @param[in] port	The serial port name (eg. 'COM1' on Windows and '/dev/ttyS0' on Linux).
 * @param[out] deviceh The device handle.
 * @return	OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_DeviceOpen(const char *port, uint32_t *deviceh);

/*!
 * Detect available devices and open them, returning an handle for each device.
 * @param[in] max max number of devices to detect.
 * @param[out] devicesh Array of handles to the detected devices.
 * @param[out] detected_num numberof detected devices.
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_DevicesDetect(uint32_t max,  uint32_t *devicesh, uint32_t *detected_num);

/*!
 * Detect the first available device and open it, returning a handle for it.
 * @param[out] deviceh The device handle.
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_DeviceDetectSingle(uint32_t *deviceh);

/*!
 * Close the specified device.
 * @param[in] deviceh A valid device handle.
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_DeviceClose(uint32_t deviceh);

/*!
 * Close all the opened devices.
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_DevicesCloseAll(void);

/**
 * @}
 * @defgroup Device Device information
 * @{
*/
/*!
 * Get the Port Name of the specified device.
 * @param[in] deviceh A valid device handle.
 * @param[out] port The serial port name (eg. 'COM1' on Windows and '/dev/ttyS0' on Linux).
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_DeviceGetPortName(uint32_t deviceh, char *port);


/*!
 * Get the connection status of the current device.
 * @param[in] deviceh A valid device handle.
 * @param[out] conn A value describing the connection status (0=off, 1=0n)
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_DeviceConnectionStatus(uint32_t deviceh, uint32_t *conn);

/**
 * @}
 * @defgroup PropInfo Property information
 * @{
*/

/*!
 * Get the number of available properties of the current device.
 * @param[in] deviceh A valid device handle.
 * @param[out] num The total number of properties for the current device.
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_PropertiesGetNumber(uint32_t deviceh, uint32_t *num);

/*!
 * Get the name of the specified property, using its index.
 * Index is zero-based, so the first property has index 0 and the last num-1 (@ref oko_PropertiesGetNumber=
 * This should be used at start to obtain the property names.
 * @param[in] deviceh A valid device handle.
 * @param[in] index Index of the property. Index is zero-based.
 * @param[out] name The name of the property.
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_PropertyGetName(uint32_t deviceh, uint32_t index, char *name);

/**
*  @typedef oko_prop_type
* Specifies the available property types
**/
typedef enum oko_prop_type
{
    OKO_PROP_UNDEF  = 0,	//!< Undefined property type
    OKO_PROP_STRING = 1,	//!< String property type
    OKO_PROP_INT   	= 2,	//!< Integer (long) property type
    OKO_PROP_DOUBLE = 3,	//!< Double property type
} oko_prop_type;


/*!
 * Get the type of the specified property.
 * @param[in] deviceh A valid device handle.
 * @param[in] name The property name.
 * @param[out] prop_type The property type as defined in @ref oko_prop_type.
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_PropertyGetType(uint32_t deviceh, const char *name, oko_prop_type *prop_type);

/*!
 * Get the measure unit of the specified property.
 * @param[in] deviceh A valid device handle.
 * @param[in] name The property name.
 * @param[out] unit Unit of measure.
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_PropertyGetUnit(uint32_t deviceh, const char *name, char *unit);

/*!
 * Get the description of the specified property.
 * @param[in] deviceh A valid device handle.
 * @param[in] name The property name.
 * @param[out] desc Description.
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_PropertyGetDescription(uint32_t deviceh, const char *name, char *desc);

/*!
 * Verify if the specified property can be write.
 * @param[in] deviceh A valid device handle.
 * @param[in] name The property name.
 * @param[out] read_only TRUE if the property is read-only.
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_PropertyGetReadOnly(uint32_t deviceh, const char *name, bool *read_only);

/*!
 * Verify if the specified property has readable limits.
 * It can be used with write properties to know if the
 * high and low value can be requested to the device.
 * @param[in] deviceh A valid device handle.
 * @param[in] name The property name.
 * @param[out] has_limits TRUE if the property limits can be read.
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_PropertyHasLimits(uint32_t deviceh, const char *name, bool *has_limits);

/*!
 * Get the minimum and maximum value that can be set for the specified property.
 * Values are zero if the property has no limits.
 * @param[in] deviceh A valid device handle.
 * @param[in] name The property name.
 * @param[out] min Minimum value that can be set.
 * @param[out] max Maximum value that can be set.
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_PropertyGetLimits(uint32_t deviceh, const char *name, double *min, double *max);

/*!
 * Verify if the specified property is a main feature of the device.
 * @param[in] deviceh A valid device handle.
 * @param[in] name The property name.
 * @param[out] is_main TRUE if the property is one of the main feature.
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_PropertyIsMain(uint32_t deviceh, const char *name, bool *is_main);

/*!
 * Verify if the specified property is an advanced feature of the device.
 * Advanced feature should are mainly used for advanced
 * application or debugging.
 * @param[in] deviceh A valid device handle.
 * @param[in] name The property name.
 * @param[out] is_adv TRUE if the property is an advanced feature.
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_PropertyIsAdvanced(uint32_t deviceh, const char *name, bool *is_adv);

/**
 * @}
 * @defgroup PropRead Reading property value
 * @{
*/


/*!
 * Get the current value of a string property.
 * @param[in] name The property name.
 * @param[in] deviceh A valid device handle.
 * @param[out] val Current value of the specified property.
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_PropertyReadString(uint32_t deviceh, const char *name, char *val);

/*!
 * Get the current value of a integer property.
 * @param[in] deviceh A valid device handle.
 * @param[in] name The property name.
 * @param[out] val Current value of the specified property.
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_PropertyReadInt(uint32_t deviceh, const char *name, int32_t *val);

/*!
 * Get the current value of a double property.
 * @param[in] deviceh A valid device handle.
 * @param[in] name The property name.
 * @param[out] val Current value of the specified property.
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_PropertyReadDouble(uint32_t deviceh, const char *name, double *val);

/*!
 * Update the specified property value, reading it from the current device.
 * @param[in] deviceh A valid device handle.
 * @param[in] name The property name.
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_PropertyUpdate(uint32_t deviceh, const char *name);  // refresh, read ?

/**
 * @}
 * @defgroup PropWrite Changing property value
 * @{
*/

/*!
 * Change the current value of the specified string property,
 *
 * @param[in] deviceh A valid device handle.
 * @param[in] name The property name.
 * @param[in] val Value to set.
 * @param[in] async (1/0) set asynchronous mode on/off (If 1 then the value will be wrote as soon as possible but not now)
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_PropertyWriteString(uint32_t deviceh, const char *name, const char *val, char async);

/*!
 * Change the current value of the specified integer property.
 *
 * @param[in] deviceh A valid device handle.
 * @param[in] name The property name.
 * @param[in] val Value to set.
 * @param[in] async (1/0) set asynchronous mode on/off (If 1 then the value will be wrote as soon as possible but not now)
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_PropertyWriteInt(uint32_t deviceh, const char *name, int32_t val, char async);

/*!
 * Change the current value of the specified double property.
 *
 * @param[in] deviceh A valid device handle.
 * @param[in] name The property name.
 * @param[in] val Value to set.
 * @param[in] async (1/0) set asynchronous mode on/off (If 1 then the value will be wrote as soon as possible but not now)
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_PropertyWriteDouble(uint32_t deviceh, const char *name, double val, char async);



/**
 * @}
 * @defgroup PropAuto Auto-updating properties
 * @{
*/

/*!\
 * Set the update frequency to read the specified property value from the current device.
 * @param[in] deviceh A valid device handle.
 * @param[in] name The property name.
 * @param[in] freq_ms The milliseconds between each property update.
 * @return OKO_OK upon success, an error code otherwise.
 */
OKO_API oko_res_type OKO_EXPORT
oko_PropertyAutoUpdate(uint32_t deviceh, const char *name, uint32_t freq_ms);


/**
 * @}
*/

#ifdef __cplusplus
}
#endif

#endif /* _OKOLIB_H_ */

